/* crypto/request.c */
/*
 * OCSP responder
 * by Massimiliano Pala (madwolf@openca.org)
 * OpenCA project 2000
 *
 * Copyright (c) 1999 The OpenSSL Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the OpenSSL Project
 *    for use in the OpenSSL Toolkit. (http://www.OpenSSL.org/)"
 *
 * 4. The names "OpenSSL Toolkit" and "OpenSSL Project" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    licensing@OpenSSL.org.
 *
 * 5. Products derived from this software may not be called "OpenSSL"
 *    nor may "OpenSSL" appear in their names without prior written
 *    permission of the OpenSSL Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the OpenSSL Project
 *    for use in the OpenSSL Toolkit (http://www.OpenSSL.org/)"
 *
 * THIS SOFTWARE IS PROVIDED BY THE OpenSSL PROJECT ``AS IS'' AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE OpenSSL PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 * This product includes cryptographic software written by Eric Young
 * (eay@cryptsoft.com).  This product includes software written by Tim
 * Hudson (tjh@cryptsoft.com).
 *
 */

#include <openssl/bio.h>
#include <openssl/err.h>
#include <openssl/rsa.h>
#include <openssl/evp.h>
#include <openssl/x509.h>
#include <openssl/x509v3.h>
#include <openssl/objects.h>
#include <openssl/pem.h>
#include <openssl/asn1.h>
#include <openssl/txt_db.h>

#include <ocsp_data.h>
#include <ocsp_req_st.h>

#undef BSIZE
#define BSIZE 256


/* Generate a new OCSP Request */
OCSP_REQUEST *generateNewRequest( X509 *cert, X509 *cacert, EVP_PKEY *pkey,
	       			EVP_MD *ocsp_digest, char *certsfile,
			       	int sign ) {
	OCSP_REQUEST 	*ret = NULL;
	X509_NAME 	*name = NULL;
	BIO 		*bio_certs = NULL;
	OCSP_CERTID 	*cid;
        X509 		*c1 = NULL;
        X509 		*c2 = NULL;
	char 		buf[256];
	STACK_OF(X509)	*sk = NULL;

	if ((ret=(OCSP_REQUEST *)OPENSSL_malloc(sizeof(OCSP_REQUEST)))==NULL )
		goto err;

	/* Let's allocate Memory for the REQINFO structure */
	ret->tbsRequest = OCSP_REQINFO_new();
	ret->optionalSignature = NULL;

	/* Get the Requestor Subject Name from passed certificate */
	if( cert )
		name = X509_get_subject_name(cert);

	if (name)
        {
		if(verbose) {
			X509_NAME_oneline(name,buf,256);
			BIO_printf(bio_err,"Certificate Subject: %s\n", buf );
		}

                if (!(ret->tbsRequest->requestorName=GENERAL_NAME_new()))
                        goto err;
                ret->tbsRequest->requestorName->type = GEN_DIRNAME;
                ret->tbsRequest->requestorName->d.dirn = X509_NAME_dup(name);
        }

        if (!(ret->tbsRequest->requestList = sk_new(NULL)))
		goto err;

	if (certsfile)
                {

		/* Read certificates from certsfile */
		bio_certs = BIO_new(BIO_s_file());
		bio_certs = BIO_new_file( certsfile, "r" );

		c1 = PEM_read_bio_X509( bio_certs, NULL,NULL, NULL );
		while ( c1 ) {
			c2 = X509_dup( c1 );

			if (!(cid = OCSP_cert_id_new(ocsp_digest,
                                             X509_get_issuer_name(c2),
                                             c2->cert_info->key->public_key,
                                             X509_get_serialNumber(c2))))
                        	goto err;

			if (!(OCSP_request_add(ret, cid, NULL))) {
				BIO_printf(bio_err,"ERROR: error adding request\n");
				goto err;
			}

			c1 = PEM_read_bio_X509( bio_certs, NULL,NULL, NULL );
		}
		
	}

	if( sign ) {
        	if (!(sk = sk_new(NULL))) goto err;
		if (cert && !sk_push(sk, (char*) cert)) goto err;
		if (cacert && !sk_push(sk, (char*) cacert)) goto err;
		OCSP_request_sign( ret, pkey, ocsp_digest, sk );
	}

	return(ret);

err:
	if(!ret) {
		BIO_printf(bio_err,"ERROR: memory allocation error(RET)\n");
	}
	if(!cid) {
		BIO_printf(bio_err,"ERROR: memory allocation error(CID)\n");
	}

	return;
}

